package ink.wgink.login.base.service;

import com.alibaba.fastjson.JSONObject;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.group.IGroupUserBaseService;
import ink.wgink.interfaces.permission.IPermissionBaseService;
import ink.wgink.interfaces.position.IPositionUserBaseService;
import ink.wgink.interfaces.role.IRoleMenuBaseService;
import ink.wgink.interfaces.role.IRolePermissionBaseService;
import ink.wgink.interfaces.role.IRoleUserBaseService;
import ink.wgink.login.base.exceptions.UserAuthenticationException;
import ink.wgink.pojo.bos.LoginUser;
import ink.wgink.pojo.bos.RoleGrantedAuthorityBO;
import ink.wgink.pojo.pos.DepartmentPO;
import ink.wgink.pojo.pos.GroupPO;
import ink.wgink.pojo.pos.PositionPO;
import ink.wgink.pojo.pos.RolePO;
import ink.wgink.service.department.service.IDepartmentUserService;
import ink.wgink.service.user.pojo.pos.UserPO;
import ink.wgink.service.user.service.IUserService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * @ClassName: UserDetailServiceImpl
 * @Description: User Detail
 * @Author: WangGeng
 * @Date: 2019/2/18 3:06 PM
 * @Version: 1.0
 **/
@Service
public class UserDetailServiceImpl implements UserDetailsService {

    private static final Logger LOG = LoggerFactory.getLogger(UserDetailServiceImpl.class);

    @Autowired(required = false)
    private IRoleUserBaseService roleUserBaseService;
    @Autowired(required = false)
    private IRoleMenuBaseService roleMenuBaseService;
    @Autowired(required = false)
    private IRolePermissionBaseService rolePermissionService;
    @Autowired(required = false)
    private IGroupUserBaseService groupUserBaseService;
    @Autowired(required = false)
    private IPositionUserBaseService positionUserBaseService;
    @Autowired
    private IDepartmentUserService departmentUserService;
    @Autowired
    private IUserService userService;

    public UserDetailServiceImpl() {
        super();
    }

    @Override
    public UserDetails loadUserByUsername(String userInfo) throws UsernameNotFoundException {
        String username;
        try {
            JSONObject userInfoObject = JSONObject.parseObject(userInfo);
            username = userInfoObject.getString("userUsername");
        } catch (Exception e) {
            username = userInfo;
        }

        UserPO userPO = userService.getPOByUsername(username.toLowerCase());
        if (null == userPO) {
            throw new UserAuthenticationException("账号不存在");
        }
        if (userPO.getUserType() != 1) {
            throw new UserAuthenticationException("非系统用户，无法登录");
        }
        if (userPO.getUserState() == 1) {
            throw new UserAuthenticationException("账号已冻结");
        }
        if (userPO.getUserState() == 2) {
            throw new UserAuthenticationException("账号已锁定");
        }

        LOG.debug("设置权限权限");
        LoginUser loginUser;
        Set<GrantedAuthority> grantedAuthorities = new LinkedHashSet<>();
        if (ISystemConstant.ADMIN.equals(username)) {
            grantedAuthorities.add(new RoleGrantedAuthorityBO(ISystemConstant.ADMIN));
            loginUser = createUserBO(userPO, grantedAuthorities);
        } else {

            List<RolePO> rolePOs = null;
            // 先处理授权，再创建对象，否则 grantedAuthorities 中无法赋值
            if (roleUserBaseService != null) {
                LOG.debug("设置角色信息");
                rolePOs = roleUserBaseService.listRolePOByUserId(userPO.getUserId());
                removalDuplicateRole(rolePOs);
                grantedAuthorities.addAll(loadRoleAuthority(rolePOs));
                if (grantedAuthorities.isEmpty()) {
                    throw new UserAuthenticationException(username + "用户无任角色");
                }
            }

            loginUser = createUserBO(userPO, grantedAuthorities);
            if (rolePOs != null) {
                loginUser.setRoles(rolePOs);
            }

            LOG.debug("设置部门状态");
            List<DepartmentPO> departmentPOs = departmentUserService.listDepartmentPOBYUserId(userPO.getUserId());
            loginUser.setDepartments(departmentPOs);

            if (groupUserBaseService != null) {
                LOG.debug("设置组信息");
                List<GroupPO> groupPOs = groupUserBaseService.listGroupPOByUserId(userPO.getUserId());
                loginUser.setGroups(groupPOs);
            }
            if (positionUserBaseService != null) {
                LOG.debug("设置职位");
                List<PositionPO> positionBOs = positionUserBaseService.listPositionPOByUserId(userPO.getUserId());
                loginUser.setPositions(positionBOs);
            }
        }
        return loginUser;
    }

    /**
     * 创建UserBO
     *
     * @param userPO
     * @param grantedAuthorities
     * @return
     */
    private LoginUser createUserBO(UserPO userPO, Set<GrantedAuthority> grantedAuthorities) {
        LoginUser loginUser = new LoginUser(userPO.getUserUsername(), userPO.getUserPassword(), userPO.getUserState() == 0 ? true : false, true, true, true, grantedAuthorities);
        loginUser.setUserId(userPO.getUserId());
        loginUser.setUserName(userPO.getUserName());
        loginUser.setUserPhone(userPO.getUserPhone());
        loginUser.setUserUKey(userPO.getUserUKey());
        loginUser.setLoginType(userPO.getLoginType());
        loginUser.setUserEmail(userPO.getUserEmail());
        loginUser.setUserAvatar(userPO.getUserAvatar());
        return loginUser;
    }

    /**
     * 角色授权列表
     *
     * @param rolePOs
     * @return
     */
    private List<GrantedAuthority> loadRoleAuthority(List<RolePO> rolePOs) {
        List<GrantedAuthority> grantedAuthorities = new ArrayList<>();
        if (null != rolePOs && !rolePOs.isEmpty()) {
            rolePOs.forEach((rolePO) -> {
                List<String> menuUrls = roleMenuBaseService.listMenuUrl(rolePO.getRoleId());
                List<String> inserts = rolePermissionService.listPermissionUrl(rolePO.getRoleId(), IPermissionBaseService.PERMISSION_INSERT);
                List<String> deletes = rolePermissionService.listPermissionUrl(rolePO.getRoleId(), IPermissionBaseService.PERMISSION_DELETE);
                List<String> updates = rolePermissionService.listPermissionUrl(rolePO.getRoleId(), IPermissionBaseService.PERMISSION_UPDATE);
                List<String> queries = rolePermissionService.listPermissionUrl(rolePO.getRoleId(), IPermissionBaseService.PERMISSION_QUERY);
                grantedAuthorities.add(new RoleGrantedAuthorityBO(rolePO.getRoleId(), rolePO.getRoleName(), menuUrls, inserts, deletes, updates, queries));
            });
        }
        return grantedAuthorities;
    }

    /**
     * 删除重复的角色
     *
     * @param rolePOs
     */
    private void removalDuplicateRole(List<RolePO> rolePOs) {
        for (int i = 0; i < rolePOs.size(); i++) {
            RolePO rolePO = rolePOs.get(i);
            boolean isExist = false;
            for (int j = i + 1; j < rolePOs.size(); j++) {
                if (StringUtils.equals(rolePO.getRoleId(), rolePOs.get(j).getRoleId())) {
                    isExist = true;
                    break;
                }
            }
            if (isExist) {
                rolePOs.remove(i);
                i--;
                break;
            }
        }
    }

}
